const std = @import("std");
const log = std.log.scoped(.pgz);
const ByteArrayList = std.ArrayList(u8);
const ProtocolError = @import("../main.zig").ProtocolError;
const ClientError = @import("../main.zig").ClientError;

const BackendKeyData = @This();
pub const Tag: u8 = 'K';

process_id: u32,
secret_key: u32,

pub fn read(a: std.mem.Allocator, buf: []const u8) !BackendKeyData {
    defer a.free(buf);
    if (buf.len != 8) return ProtocolError.InvalidMessageLength;
    return .{
        .process_id = std.mem.readIntBig(u32, buf[0..4]),
        .secret_key = std.mem.readIntBig(u32, buf[4..8]),
    };
}
pub fn write(self: BackendKeyData, _: std.mem.Allocator, stream_writer: anytype) !void {
    try stream_writer.writeByte(Tag);
    try stream_writer.writeIntBig(u32, 12); // length
    try stream_writer.writeIntBig(u32, self.process_id);
    try stream_writer.writeIntBig(u32, self.secret_key);
}
pub fn deinit(_: *BackendKeyData, _: std.mem.Allocator) void {}

test "round trip" {
    const allocator = std.testing.allocator;
    var sm = BackendKeyData{
        .process_id = 123,
        .secret_key = 345,
    };
    defer sm.deinit(allocator);

    var bal = ByteArrayList.init(allocator);
    defer bal.deinit();
    try sm.write(allocator, bal.writer());

    var fbs = std.io.fixedBufferStream(bal.items);
    var reader = fbs.reader();
    const tag = try reader.readByte();
    try std.testing.expectEqual(Tag, tag);
    const len = try reader.readIntBig(u32);
    const buf = try allocator.alloc(u8, len - 4);
    try reader.readNoEof(buf);
    var sm2 = try BackendKeyData.read(allocator, buf);
    defer sm2.deinit(allocator);
    try std.testing.expectEqual(@as(u32, 123), sm2.process_id);
    try std.testing.expectEqual(@as(u32, 345), sm2.secret_key);
}
