const std = @import("std");
const log = std.log.scoped(.pgz);
const ByteArrayList = std.ArrayList(u8);
const ProtocolError = @import("../main.zig").ProtocolError;
const ClientError = @import("../main.zig").ClientError;
const enum_from_int = @import("../main.zig").enum_from_int;
const FormatCode = @import("../main.zig").FormatCode;

pub const Tag: u8 = 'C';

const CommandComplete = @This();

buf: ?[]const u8 = null, // owned
command_tag: []const u8,

pub fn read(_: std.mem.Allocator, buf: []const u8) !CommandComplete {
    return .{
        .buf = buf,
        .command_tag = buf[0..],
    };
}

pub fn write(self: CommandComplete, _: std.mem.Allocator, stream_writer: anytype) !void {
    try stream_writer.writeByte(Tag);
    try stream_writer.writeIntBig(u32, @as(u32, @intCast(4 + self.command_tag.len)));
    try stream_writer.writeAll(self.command_tag);
}

pub fn deinit(self: *CommandComplete, a: std.mem.Allocator) void {
    if (self.buf != null) a.free(self.buf.?);
}

pub fn clone(self: CommandComplete, a: std.mem.Allocator) !CommandComplete {
    var ba = ByteArrayList.init(a);
    errdefer ba.deinit();
    try self.write(a, ba.writer());
    return try CommandComplete.read(a, ba.items);
}

test "round trip" {
    const allocator = std.testing.allocator;
    var sm = CommandComplete{
        .command_tag = "foo",
    };
    defer sm.deinit(allocator);

    var bal = ByteArrayList.init(allocator);
    defer bal.deinit();
    try sm.write(allocator, bal.writer());

    var fbs = std.io.fixedBufferStream(bal.items);
    var reader = fbs.reader();
    const tag = try reader.readByte();
    try std.testing.expectEqual(Tag, tag);
    const len = try reader.readIntBig(u32);
    const buf = try allocator.alloc(u8, len - 4);
    try reader.readNoEof(buf);
    var sm2 = try CommandComplete.read(allocator, buf);
    defer sm2.deinit(allocator);

    try std.testing.expectEqualStrings("foo", sm2.command_tag);
}
