const std = @import("std");
const log = std.log.scoped(.pgz);
const ByteArrayList = std.ArrayList(u8);
const ProtocolError = @import("../main.zig").ProtocolError;
const ClientError = @import("../main.zig").ClientError;

const ParameterStatus = @This();
pub const Tag: u8 = 'S';

buf: ?[]const u8 = null, // owned
name: []const u8,
value: []const u8,

pub fn read(a: std.mem.Allocator, buf: []const u8) !ParameterStatus {
    errdefer a.free(buf);
    var it = std.mem.splitScalar(u8, buf, 0);
    return .{
        .buf = buf,
        .name = it.first(),
        .value = it.next() orelse return ProtocolError.MissingField,
    };
}
pub fn write(self: ParameterStatus, a: std.mem.Allocator, stream_writer: anytype) !void {
    try stream_writer.writeByte(Tag);
    var al = ByteArrayList.init(a);
    defer al.deinit();
    var cw = std.io.countingWriter(al.writer());
    var writer = cw.writer();
    try writer.writeIntBig(u32, 0); // length placeholder
    try writer.writeAll(self.name);
    try writer.writeByte(0);
    try writer.writeAll(self.value);
    try writer.writeByte(0);
    std.mem.writeIntBig(u32, al.items[0..4], @as(u32, @intCast(cw.bytes_written))); // Fix length
    try stream_writer.writeAll(al.items);
}
pub fn deinit(self: *ParameterStatus, allocator: std.mem.Allocator) void {
    if (self.buf != null) allocator.free(self.buf.?);
}

test "round trip" {
    const allocator = std.testing.allocator;
    var sm = ParameterStatus{
        .name = "Hello",
        .value = "world",
    };
    defer sm.deinit(allocator);

    var bal = ByteArrayList.init(allocator);
    defer bal.deinit();
    try sm.write(allocator, bal.writer());

    var fbs = std.io.fixedBufferStream(bal.items);
    var reader = fbs.reader();
    const tag = try reader.readByte();
    try std.testing.expectEqual(Tag, tag);
    const len = try reader.readIntBig(u32);
    const buf = try allocator.alloc(u8, len - 4);
    try reader.readNoEof(buf);
    var sm2 = try ParameterStatus.read(allocator, buf);
    defer sm2.deinit(allocator);
    try std.testing.expectEqualStrings("Hello", sm2.name);
    try std.testing.expectEqualStrings("world", sm2.value);
}
