const std = @import("std");
const ByteArrayList = std.ArrayList(u8);
const PasswordMessage = @This();

pub const Tag: u8 = 'p';
password: []const u8,
password_owned: bool = false,

pub fn read(allocator: std.mem.Allocator, b: []const u8) !PasswordMessage {
    return .{ .password = try allocator.dupe(u8, b), .password_owned = true };
}

pub fn write(self: PasswordMessage, _: std.mem.Allocator, stream_writer: anytype) !void {
    try stream_writer.writeByte(Tag);
    try stream_writer.writeIntBig(u32, 5 + @as(u32, @intCast(self.password.len)));
    try stream_writer.writeAll(self.password);
    try stream_writer.writeByte(0);
}

pub fn deinit(self: *PasswordMessage, allocator: std.mem.Allocator) void {
    if (self.password_owned) allocator.free(self.password);
}

test "round trip" {
    const allocator = std.testing.allocator;
    var sm = PasswordMessage{
        .password = "foobar",
    };
    defer sm.deinit(allocator);

    var bal = ByteArrayList.init(allocator);
    defer bal.deinit();
    try sm.write(allocator, bal.writer());

    var fbs = std.io.fixedBufferStream(bal.items);
    var reader = fbs.reader();
    const tag = try reader.readByte();
    try std.testing.expectEqual(Tag, tag);
    const len = try reader.readIntBig(u32);
    const buf = try allocator.alloc(u8, len - 4);
    defer allocator.free(buf);
    try reader.readNoEof(buf);
    var sm2 = try PasswordMessage.read(allocator, buf);
    defer sm2.deinit(allocator);
}
